@@ -6,8 +6,9 @@
 # David Hunter <dhunter@stat.psu.edu> and Mark S. Handcock
 # <handcock@u.washington.edu>.
 #
-# Last Modified 02/26/13
+# Last Modified 06/06/21
 # Licensed under the GNU General Public License version 2 (June, 1991)
+# or greater
 #
 # Part of the R/network package
 #
@@ -16,28 +17,22 @@
 #
 # Contents:
 #
-#   as.color
-#   mixingmatrix
-#   network.density
-#   is.color
 #   is.discrete
 #   is.discrete.character
 #   is.discrete.numeric
-#   print.mixingmatrix
-#   which.matrix.type
 #
 ######################################################################
 
-
-#Given a vector of non-colors, try to coerce them into some reasonable
-#color format.  This may not work well, but what the hell....
-
-
 #' Transform vector of values into color specification
 #' 
 #' Convenience function to convert a vector of values into a color
 #' specification.
 #' 
+#' @param x vector of numeric, character or factor values to be transformed
+#' @param opacity optional numeric value in the range 0.0 to 1.0 used to specify
+#'   the opacity/transparency (alpha) of the colors to be returned. 0 means
+#'   fully opaque, 1 means fully transparent.
+#' 
 #' Behavior of \code{as.color} is as follows: \itemize{ \item integer numeric
 #' values: unchanged, (assumed to corespond to values of R's active
 #' \code{\link{palette}}) \item integer real values: will be translated to into
@@ -59,13 +54,13 @@
 #' These functions are used for the color parameters of
 #' \code{\link{plot.network}}.
 #' 
-#' @param x vector of numeric, character or factor values to be transformed
-#' @param opacity optional numeric value in the range 0.0 to 1.0 used to
-#' specify the opacity/transparency (alpha) of the colors to be returned. 0
-#' means fully opaque, 1 means fully transparent.
 #' @return For \code{as.color}, a vector integer values (corresponding to color
-#' palette values) or character color name. For \code{is.color}, a logical
-#' vector indicating if each element of x appears to be a color
+#'   palette values) or character color name. For \code{is.color}, a logical
+#'   vector indicating if each element of x appears to be a color
+#' 
+#' @rdname as.color
+#' @export
+#' 
 #' @examples
 #' 
 #' 
@@ -76,8 +71,6 @@
 #' as.color(c('red','green','blue'),0.5) # gives "#FF000080", "#00FF0080", "#0000FF80"
 #' 
 #' is.color(c('red',1,'foo',NA,'#FFFFFF55'))
-#' 
-#' @export as.color
 as.color<-function(x,opacity=1.0){
   if(opacity > 1 | opacity < 0){
     stop('opacity parameter must be a numeric value in the range 0 to 1')
@@ -110,48 +103,86 @@ as.color<-function(x,opacity=1.0){
 }
 
 
-#Return the mixing matrix for a network object, on a given attribute.  This
-#is a relocated function from the ergm package; it probably belongs elsewhere,
-#but is needed for the summary.network method (and in that sense is basic
-#enough to include.
-#' Internal Network Package Functions
+# Mixing matrix -----------------------------------------------------------
+
+#' Mixing matrix
 #' 
-#' Internal network functions.
+#' Return the mixing matrix for a network, on a given attribute.
 #' 
-#' Most of these are not to be called by the user.
-#' 
-#' @name network-internal
-#' 
-#' @aliases + - * +.default -.default *.default summary.character
-#' print.summary.character print.mixingmatrix
 #' @param object a network or some other data structure for which a mixing
-#' matrix is meaningful.
-#' @param x an object to be designated either discrete or continuous, or a
-#' network.
-#' @param attrname a vertex attribute name.
-#' @param y a network or something coercible to one.
-#' @param \dots further arguments passed to or used by methods.
-#' @seealso network
-#' @keywords internal
+#'   matrix is meaningful.
+#' @param ... further arguments passed to or used by methods.
+#' 
+#' @rdname mixingmatrix
 #' @export
+
 mixingmatrix <- function(object, ...) UseMethod("mixingmatrix")
 
-#' @rdname network-internal
+
+# Return the mixing matrix for a network object, on a given attribute.  This is
+# a relocated function from the ergm package; it probably belongs elsewhere, but
+# is needed for the summary.network method (and in that sense is basic enough to
+# include.
+
+
+
+
+#' @rdname mixingmatrix
+#'
+#' @param attrname a vertex attribute name.
+#' @param expand.bipartite logical; if `object` is bipartite, should we return
+#'   the *square* mixing matrix representing every level of `attrname` against
+#'   every other level, or a *rectangular* matrix considering only levels
+#'   present in each bipartition?
+#' @param useNA one of "ifany", "no" or "always". Argument passed to
+#'   \code{\link{table}}. By default (\code{useNA = "ifany"}) if there are any
+#'   \code{NA}s on the attribute corresponding row \emph{and} column will be
+#'   contained in the result. See Details.
+#' @param ... arguments passed to \code{\link{table}}.
+#'
+#' @details Handling of missing values on the attribute \code{attrname} almost
+#'   follows similar logic to \code{\link{table}}. If there are \code{NA}s on
+#'   the attribute and \code{useNA="ifany"} (default) the result will contain
+#'   both row and column for the missing values to ensure the resulting matrix
+#'   is square (essentially calling \code{\link{table}} with
+#'   \code{useNA="always"}). Also for that reason passing \code{exclude}
+#'   parameter with \code{NULL}, \code{NA} or \code{NaN} is ignored with a
+#'   warning as it may break the symmetry.
+#'
+#' @return Function `mixingmatrix()` returns an object of class `mixingmatrix`
+#'   extending `table` with a cross-tabulation of edges in the `object`
+#'   according to the values of attribute `attrname` for the two incident
+#'   vertices. If `object` is a *directed* network rows correspond to the "tie
+#'   sender" and columns to the "tie receiver". If `object` is an *undirected*
+#'   network there is no such distinction and the matrix is symmetrized. In both
+#'   cases the matrix is square and all the observed values of the attribute
+#'   `attrname` are represented in rows and columns. If `object` is a
+#'   *bipartite* network and `expand.bipartite` is `FALSE` the resulting matrix
+#'   does not have to be square as only the actually observed values of the
+#'   attribute are shown for each partition, if `expand.bipartite` is `TRUE` the
+#'   matrix will be square.
+#'
 #' @export
-mixingmatrix.network <- function(object, attrname, ...) {
+#' @examples
+#' # Interaction ties between Lake Pomona SAR organizations by sponsorship type
+#' # of tie sender and receiver (data from Drabek et al. 1981)
+#' data(emon)
+#' mixingmatrix(emon$LakePomona, "Sponsorship")
+mixingmatrix.network <- function(object, attrname, useNA = "ifany", expand.bipartite=FALSE, ...) {
   nw <- object
   if(missing(attrname)){
     stop("attrname argument is missing. mixingmatrix() requires an an attribute name")
   }
-  if(network.size(nw)==0){
+  if(!(attrname %in% list.vertex.attributes(object)))
+    stop("vertex attribute ", sQuote(attrname), " not found in network ",
+         sQuote(deparse(substitute(object))))
+  if(network.size(nw)==0L){
     warning("mixing matrices not well-defined for graphs with no vertices.")
-    type<-"directed"
-    if(is.bipartite(nw))
-      type<-"bipartite"
-    tabu<-matrix(nrow=0,ncol=0)
-    ans<-list(type=type,matrix=tabu)
-    class(ans)<-"mixingmatrix"
-    return(ans)
+    return(as.mixingmatrix(
+      matrix(nrow=0L, ncol=0L),
+      directed = is.directed(object),
+      bipartite = is.bipartite(object)
+    ))
   }
   nodecov <- unlist(get.vertex.attribute(nw, attrname))
   u<-sort(unique(nodecov))
@@ -163,36 +194,141 @@ mixingmatrix.network <- function(object, attrname, ...) {
       cat("Warning:  Bipartite networks are currently\n",
           "automatically treated as undirected\n")
     type <- "bipartite"
-    rowswitch <- apply(el, 1, function(x) x[1]>x[2])
-    el[rowswitch, 1:2] <- el[rowswitch, 2:1]
+    rowswitch <- apply(el, 1L, function(x) x[1L]>x[2L])
+    el[rowswitch, 1L:2L] <- el[rowswitch, 2L:1L]
     nb1 <- get.network.attribute(nw,"bipartite")
-    u<-sort(unique(nodecov[0:nb1]))
-    From <- c(u, nodecov[el[,1]])
-    u<-sort(unique(nodecov[(nb1+1):network.size(nw)]))
-    To <- c(u, nodecov[el[,2]])
+    if(!expand.bipartite) u <- sort(unique(nodecov[1L:nb1]))
+    From <- factor(nodecov[el[,1L]], levels=u)
+    if(!expand.bipartite) u <- sort(unique(nodecov[(nb1+1L):network.size(nw)]))
+    To <- factor(nodecov[el[,2L]], levels=u)
   }else{
-    From <- c(u, nodecov[el[,1]])
-    To <- c(u, nodecov[el[,2]])
+    From <- factor(nodecov[el[,1L]], levels=u)
+    To <- factor(nodecov[el[,2L]], levels=u)
   }
-  tabu <- table(From, To)  # Add u,u diagonal to ensure each 
-  # value is represented, then subtract it later
-  diag(tabu) <- diag(tabu) - 1
+  if(any(is.na(nodecov)) && useNA == "ifany") useNA <- "always"
+  dots <- list(...)
+  if("exclude" %in% names(dots) && (is.null(dots$exclude) | any(is.na(dots$exclude)) | any(is.nan(dots$exclude)))) {
+    warning("passing `exclude=NULL` to table() is not supported, ignoring")
+    dots$exclude <- NULL
+  }
+  tabu <- do.call(table, c(list(From=From, To=To, useNA=useNA), dots))
   if(!is.directed(nw) && !is.bipartite(nw)){
     type <- "undirected"
     tabu <- tabu + t(tabu)
-    diag(tabu) <- diag(tabu)/2
+    diag(tabu) <- diag(tabu)%/%2L
   }
-  ans <- list(type=type, matrix=tabu)
-  class(ans) <- "mixingmatrix"
-  ans
+  as.mixingmatrix(
+    tabu,
+    directed = is.directed(object),
+    bipartite = is.bipartite(object)
+  )
+}
+
+#' @rdname mixingmatrix
+#' 
+#' @note The `$` and `[[` methods are included only for backward-compatiblity
+#'   reason and will become defunct in future releases of the package.
+#' 
+#' @export
+"[[.mixingmatrix" <- function(x, ...) {
+  .Deprecated(
+    new = "mixingmatrix",
+    msg = "Mixing matrix objects now extend class \"table\". The `[[` method is deprecated and will be removed from future releases of the package. See ?mixingmatrix for details."
+  )
+  x <- .to_oldmm(x)
+  NextMethod()
 }
 
 
-# Return the density of the given network.  (This probably won't stay in
-# this package....
-#
+#' @rdname mixingmatrix
+#' 
+#' @param name name of the element to extract, one of "matrix" or "type"
+#'
+#' @export
+"$.mixingmatrix" <- function(x, name) {
+  .Deprecated(
+    new = "mixingmatrix",
+    msg = "Mixing matrix objects now extend class \"table\". The `$` method is deprecated and will be removed from future releases of the package. See ?mixingmatrix for details."
+  )
+  x <- .to_oldmm(x)
+  NextMethod()
+}
 
 
+.to_oldmm <- function(x) {
+  directed <- attr(x, "directed")
+  bipartite <- attr(x, "bipartite")
+  list(
+    matrix = structure(as.integer(x), dimnames=dimnames(x), dim=dim(x)),
+    type = if(bipartite) "bipartite" else if(directed) "directed" else "undirected"
+  )
+}
+
+
+# A non-exported constructor of mixingmatrix objects
+# 
+# @param mat matrix with the actual cross-tabulation
+# @param directed logical if the network is directed
+# @param bipartite logical if the netwoek is bipartite
+# @param ... other arguments currently ignored
+# 
+# @return The matrix with attributes `directed` and `bipartite` of class
+#   `mixingmatrix` inheriting from `table`.
+
+as.mixingmatrix <- function(mat, directed, bipartite, ...) {
+  # Test/check/symmetrize here?
+  structure(
+    mat,
+    directed = directed,
+    bipartite = bipartite,
+    class = c("mixingmatrix", "table")
+  )
+}
+
+
+#' @rdname mixingmatrix
+#' 
+#' @return Functions `is.directed()` and `is.bipartite()` return `TRUE` or
+#'   `FALSE`. The values will be identical for the input network `object`.
+#' 
+#' @export
+is.directed.mixingmatrix <- function(x, ...) attr(x, "directed")
+
+#' @rdname mixingmatrix
+#' @export
+is.bipartite.mixingmatrix <- function(x, ...) attr(x, "bipartite")
+
+
+#' @rdname mixingmatrix
+#' 
+#' @param x mixingmatrix object
+#' 
+#' @export
+print.mixingmatrix <- function(x, ...) {
+  m <- x
+  rn <- rownames(x)
+  cn <- colnames(x)  
+  if (!attr(x, "directed")) {
+    dimnames(m) <- list(rn, cn)
+    on.exit(
+      message("Note:  Marginal totals can be misleading for undirected mixing matrices.")  
+    )
+  } else {
+    dimnames(m) <- if(attr(x, "bipartite")) list(B1 = rn, B2 = cn) else list(From = rn, To = cn)
+    m <- stats::addmargins(m)
+  }
+  m <- structure(
+    m,
+    directed = attr(x, "directed"),
+    bipartite = attr(x, "bipartite"),
+    class = "table"
+  )
+  print(m)
+}
+
+
+# network.density ---------------------------------------------------------
+
 #' Compute the Density of a Network
 #' 
 #' \code{network.density} computes the density of its argument.
@@ -217,7 +353,7 @@ mixingmatrix.network <- function(object, attrname, ...) {
 #' @seealso \code{\link{network.edgecount}}, \code{\link{network.size}}
 #' @references Butts, C. T.  (2008).  \dQuote{network: a Package for Managing
 #' Relational Data in R.} \emph{Journal of Statistical Software}, 24(2).
-#' \url{http://www.jstatsoft.org/v24/i02/}
+#' \url{https://www.jstatsoft.org/v24/i02/}
 #' 
 #' Wasserman, S. and Faust, K.  (1994).  \emph{Social Network Analysis: Methods
 #' and Applications.} Cambridge: Cambridge University Press.
@@ -231,6 +367,7 @@ mixingmatrix.network <- function(object, attrname, ...) {
 #' g<-network.initialize(5)    #Initialize the network
 #' network.density(g)          #Calculate the density
 #' 
+#' @rdname network.density
 #' @export network.density
 network.density<-function(x,na.omit=TRUE,discount.bipartite=FALSE){
   if(!is.network(x))
@@ -263,8 +400,7 @@ network.density<-function(x,na.omit=TRUE,discount.bipartite=FALSE){
   ec/pe
 }
 
-# has.edges  checks if any of the specified vertex ids have edges (are not isolates)
-
+# has.edges ---------------------------------------------------------------
 
 #' Determine if specified vertices of a network have any edges (are not
 #' isolates)
@@ -286,6 +422,7 @@ network.density<-function(x,na.omit=TRUE,discount.bipartite=FALSE){
 #' has.edges(test)
 #' has.edges(test,v=5)
 #' 
+#' @rdname has.edges
 #' @export has.edges
 has.edges<-function(net,v=seq_len(network.size(net))){
   if(network.size(net)==0){
@@ -299,9 +436,12 @@ has.edges<-function(net,v=seq_len(network.size(net))){
   return(ins+outs != 0)
 }
 
+# is.color ----------------------------------------------------------------
 
-#Returns TRUE if x is a character in a known color format
 #' @rdname as.color
+#' 
+#' @return \code{as.color()} returns TRUE if x is a character in a known color format.
+#' 
 #' @export
 is.color<-function(x){
   xic<-rep(FALSE,length(x))         #Assume not a color by default
@@ -314,6 +454,25 @@ is.color<-function(x){
   xic
 }
 
+
+
+#' Internal Network Package Functions
+#' 
+#' Internal network functions.
+#' 
+#' Most of these are not to be called by the user.
+#' 
+#' @name network-internal
+#' 
+#' @param x an object to be designated either discrete or continuous, or a
+#' network.
+#' @param y a network or something coercible to one.
+#' @param \dots further arguments passed to or used by methods.
+#' 
+#' @seealso network
+#' 
+#' @keywords internal
+
 #' @rdname network-internal
 #' @export
 is.discrete.numeric<-function(x){
@@ -333,35 +492,15 @@ is.discrete<-function(x){
 }
 
 
-#Print method for mixingmatrix objects
-#' @export print.mixingmatrix
-#' @export
-print.mixingmatrix <- function(x, ...) {
-  m <- x$mat
-  rn <- rownames(m)
-  cn <- colnames(m)  
-  if (x$type == "undirected") {
-    dimnames(m) <- list(rn, cn)
-    cat("Note:  Marginal totals can be misleading\n",
-        "for undirected mixing matrices.\n")
-  } else {
-    total <- apply(m,1,sum)
-    m <- cbind(m,total)
-    total <- apply(m,2,sum)
-    m <- rbind(m,total)
-    rn <- c(rn, "Total")
-    cn <- c(cn, "Total")
-    if (x$type == "bipartite")
-      dimnames(m) <- list(B1 = rn,B2 = cn)
-    else
-      dimnames(m) <- list(From = rn,To = cn)
-  }
-  print(m)
-}
 
 
 
 
+
+
+
+# which.matrix.type -------------------------------------------------------
+
 #' Heuristic Determination of Matrix Types for Network Storage
 #' 
 #' \code{which.matrix.type} attempts to choose an appropriate matrix expression
@@ -379,7 +518,7 @@ print.mixingmatrix <- function(x, ...) {
 #' @seealso \code{\link{as.matrix.network}}, \code{\link{as.network.matrix}}
 #' @references Butts, C. T.  (2008).  \dQuote{network: a Package for Managing
 #' Relational Data in R.} \emph{Journal of Statistical Software}, 24(2).
-#' \url{http://www.jstatsoft.org/v24/i02/}
+#' \url{https://www.jstatsoft.org/v24/i02/}
 #' @keywords graphs
 #' @examples
 #' 
@@ -396,6 +535,7 @@ print.mixingmatrix <- function(x, ...) {
 #'   which.matrix.type(as.matrix.network(g,matrix.type="incidence"))
 #'   which.matrix.type(as.matrix.network(g,matrix.type="edgelist"))
 #' 
+#' @rdname which.matrix.type
 #' @export which.matrix.type
 which.matrix.type<-function(x)
 {
@@ -428,3 +568,23 @@ which.matrix.type<-function(x)
     }
     out
 }
+
+
+# This is a temporary transplant from statnet.common, copied here to ease transition on CRAN.
+#' @importFrom statnet.common EVL
+simplify_simple <- function(x, toNA = c("null","empty","keep"), empty = c("keep", "unlist"), ...){
+  if(isFALSE(toNA)) toNA <- "keep"
+  toNA <- match.arg(toNA)
+  empty <- match.arg(empty)
+
+  if(is.atomic(x)) return(x)
+
+  x <- switch(toNA,
+              keep = x,
+              null = lapply(x, NVL, NA),
+              empty = lapply(x, EVL, NA))
+
+  if(length(x)==0) switch(empty, keep=x, unlist=unlist(x, recursive=FALSE, ...))
+  else if(all(lengths(x)==1L) && all(vapply(x, is.atomic, logical(1)))) unlist(x, recursive=FALSE, ...)
+  else x
+}
